function [afM,yM]=fmGMSK_v3(fs,sps,fb,sym,ilag)
%fmGMSK_v3 modulate Gaussian Minimum Shift Keying
%   modulates a binary message stream with MSK scheme
% Version history (original function is mGMSK)
%   V1: Modded to accept filter bandwidth (fb) input
%   V2: uses refadj_v1
%   V3: refadj function is now local MSKa function;
%       MSKa function does not align to the correct quadrant, to be done outside of this function
%       inputs have been changed accordingly!
[syms]=gSs(sym,ilag,sps); [afM]=psf(syms,sps,fb,fs);
[mI,mQ]=pol2cart(cumsum(afM)*2*pi/fs,1); yM=mI+1j*mQ;
yM=MSKa(yM,ilag,sym,sps);
end

function [sym]=gSs(symr,ilag,sps)
%GSS get symbol signal
%   Detailed explanation goes here
% fprintf('number of symbols is %d\n',length(symr));   % diagnostics
sym=reshape(symr(:,ones(sps,1))',[],1);                % make all symbols equal length
sym=[zeros(ilag-1,1);sym;zeros(sps-ilag+1,1)];         % align to transition and zero pad
sym=double(sym);                                       % convert to double for subsequent operations
end

function [dpM]=psf(sym,sps,fb,fs)
%psf pulse shaping filter for VLF GMSK transmissions
%   Gaussian filters a non-return-to-zero binary bit stream to produce the
%   apparent frequency modulation of a VLF GMSK transmission
% Version history (original function is psf)
%   V1.0: Modded to accept filter bandwidth (fb) input
ns=length(sym);                                        % length of each bit and total length of bit stream
Mod_i=0.5*pi; r=sqrt(2); sf2=fb^2/2/log(r);            % filter parameters
f=[0:ns/2,ns/2-1:-1:1]'*fs/ns; Gf=exp(-f.^2/2/sf2);    % calculate Gaussian function
dpM=(ifft(fft(sym).*Gf)*Mod_i/sps)*fs/2/pi;            % IIR filter
% FIR filter ??
% span=1; bt=0.6; Mod_i=0.5*pi; h=gaussdesign(bt,span,sps);
% dpM=conv(sym,h,'valid')*Mod_i/sps;
% dpM=filter(h,1,sym)*Mod_i/sps;
end

function ya=MSKa(yM,ilag,sym,sps)
%MSKa align MSK phase to carrier
%   Detailed explanation goes here
int=[false;(sym(1:end-1)-sym(2:end))==0];              % find symbols that have not changed value from the previous
Mod_i=0.5*pi; ycM=[0;cumsum(sym*Mod_i)];               % crude MSK model with transitions on axes (phase)
[ycMi,ycMq]=pol2cart(ycM,1); ycM=ycMi+1j*ycMq;         % crude MSK model with transitions on axes (complex vector)
yMt=yM(ilag:sps:end); pda=yMt(int).*conj(ycM(int));    % phase difference between transitions and axes
ya=yM*exp(-1j*mean(angle(pda)));                       % align model so that transitions are on axes
end