% read national instruments binary data
% [y,fs,n,ir]=nibreads(datnam,ichan,nstrt,isec)
% input: 
% datnam - filename 
% ichan  - number of channels
% nstrt  - skip nstrt samples for analysis 
% isec   - number of seconds for analysis 
% output: 
% y      - data stream (amplitude in V)
% fs     - sampling frequency
% n      - number of samples 
% ir     - number of missing values 
function [y,fs,n,ir]=frnib(datnam,ichan,nstrt,isec)
clear y fs n ir;                                 % initialise 
fid=fopen(datnam,'r','ieee-be');                 % open datnam
h=fread(fid,2,'float64');                        % read header length 
icof=h(1); ichn=h(2); ilen=icof*ichn;            % determine header length
h=fread(fid,ilen,'float64');                     % read header 
fs=h(1); cof=reshape(h(h~=fs),icof-1,ichn);      % assign fs and coef's
ihlenb=(ilen+2)*8;                               % header length in byte
n=isec*fs; y(1:n,1)=32767; ir=n;                 % initialise samples to read 
fseek(fid,0,'eof'); nb=ftell(fid);               % number of bytes in file
ns=(nb-ihlenb)/2;                                % number of samples in file
if nstrt+isec*fs>ns;                             % too many samples requested 
 'end of file reached !', return;                % end of file message 
end;                                             % return to main 
iskip=round(nstrt*ichn+ichan-1);                 % skip samples and chan's 
fseek(fid,ihlenb+iskip*2,'bof');                 % position pointer
[y,icnt]=fread(fid,n,'int16',2*(ichn-1));        % read channel and skip 
if icnt~=n; y(icnt+1:n,1)=32767; end;            % add missing values
ir=y==-32768|y==32767;                           % count missing values 
%figure(1); plot(y)                              % plot original i16 values 
y=cof(1,ichan)+cof(2,ichan)*y+...                % scale to Volts
  cof(3,ichan)*y.^2+cof(4,ichan)*y.^3;           % scale to Volts 
fclose(fid);                                     % close file