% 10 ms statistical mode of radio data
% one hour from 11:30--12:30UTC
% this time period is split over 2 data files!! :(

% define some parameters
fs=1e6;         % sampling frequency (Hz)
rs=1e4;        % resolution (Samples)
datL=rs/fs;    % length of data chunk (s)

bw=1e-4; eLH=[-1 1]*1.1;            % bin width and limits
nBins=(eLH(2)-eLH(1))/bw;           % number of bins
ed=linspace(eLH(1),eLH(2),nBins+1); % bin edges
ct=ed+bw/2; ct=ct(1:end-1);         % bin centres

datN=DDT(11); fT=ffDT(datN(end-26:end));    % data file 1 starts from 11:28:43 UTC
[nl]=fnl(datN,rs);                          % number of processing blocks from data file
tS=(0:datL:datL*(nl-1)); nstrt=tS*fs;       % chunk start times (in seconds and samples)

y_mode=zeros(nl,1);

for ii=1:nl                         % iterate every data chunk
    [y,~,~,ir]=frnib(datN,1,nstrt(ii),datL);    % read data
    h=histcounts(y,ed);                         % compute distribution
    [~,im]=max(h); y_mode(ii)=ct(im);           % mode value
end

t_mode=linspace(0,length(y_mode)-1,length(y_mode))*datL;
t_mode=fT+seconds(t_mode);

datN=DDT(12); fT=ffDT(datN(end-26:end));    % data file 2 starts from 12:28:43 UTC
eTi=datetime(2016,8,27,12,30,0);
durTs=seconds(eTi-fT);
t_mode1=linspace(0,durTs,durTs/datL); t_mode1=fT+seconds(t_mode1);

y_mode1=zeros(length(t_mode1),1);
for ii=1:length(t_mode1)
    [y,~,~,ir]=frnib(datN,1,nstrt(ii),datL);    % read data
    h=histcounts(y,ed);                         % compute distribution
    [~,im]=max(h); y_mode1(ii)=ct(im);           % mode value
end

y=[y_mode;y_mode1]; t=[t_mode t_mode1];
y(abs(y)>1)=nan;

save('./Processed_data/10ms_mode.mat','y','t');